<?php
class Starrys {
	protected $registry;

	public function __construct($registry) {
		$this->registry = $registry;
		$folder = DIR_APPLICATION.'/../system/library/starrysapi/';
		require_once($folder.'client.php');
		require_once($folder.'command.php');
		require_once($folder.'devicestatus.php');
		require_once($folder.'documenttype.php');
		require_once($folder.'helper.php');
		require_once($folder.'noncashtype.php');
		require_once($folder.'payattribute.php');
		require_once($folder.'taxid.php');
		require_once($folder.'productlist.php');
	}

	public function __get($key) {
		return $this->registry->get($key);
	}

	public function __set($key, $value) {
		$this->registry->set($key, $value);
	}

	private function getType() {
		return basename(DIR_APPLICATION);
	}

	public function updateOrder($order_data) {
		$this->load->model('checkout/order');
		$order_data = $this->model_checkout_order->getOrder($order_data['order_id']);
		//file_put_contents(__DIR__.'/log.txt', var_export(array($order_data['order_status_id'], $this->config->get('starrys_statuses_print'), $this->config->get('starrys_statuses_return'),$order_data['payment_code'], $this->config->get('starrys_payment_methods')	),true));
		if(
			(
				(
					in_array($order_data['order_status_id'], $this->config->get('starrys_statuses_print'))
					|| in_array($order_data['order_status_id'], $this->config->get('starrys_statuses_return'))
				)
				&& in_array($order_data['payment_code'], $this->config->get('starrys_payment_methods'))
			)
		)
		{
			if (in_array($order_data['order_status_id'], $this->config->get('starrys_statuses_print'))){
				$type = "sell";
			} else {
				$type = "return";
			}

			if($this->config->get('starrys_log')) {
				$this->writeLog('order', $order_data);
			}
			$items = array();
			$products = $this->getOrderProducts($order_data['order_id']);
			$totals = $this->getOrderTotals($order_data['order_id']);
			$tax = 0;
			$voucher = 0;
			$shipping = 0;
			$subtotal = 0;
			$coupon = 0;
			$reward = 0;
			$credit = 0;
			$low_order_fee = 0;
			$handling = 0;
			foreach($totals as $total) {
				switch($total['code']){
					case 'tax':$tax += $total['value'];break;
					case 'shipping': $shipping += $total['value'];break;
					case 'sub_total': $subtotal += $total['value'];break;
					case 'coupon': $coupon += $total['value'];break;
					case 'voucher': $voucher += $total['value'];break;
					case 'reward': $reward += $total['value'];break;
					case 'credit': $credit += $total['value'];break;
					case 'handling': $handling += $total['value'];break;
					case 'low_order_fee': $low_order_fee += $total['value'];break;
				}
			}

			$additional = ($tax + $coupon + $voucher + $reward + $credit + $handling + $low_order_fee)/$subtotal;

			if($this->config->get('starrys_log')) {
				$this->writeLog('items', $products);
			}
			$checkId = $this->getCheckId($order_data['order_id'], $type);
			$productCommands = array();
			if ($type == 'sell') {
				$documentType = Starrys\Cashbox\Api\DocumentType::DEBIT;
			} else {
				$documentType = Starrys\Cashbox\Api\DocumentType::REFUND_DEBIT;
			}
			$vat = intval($this->config->get('starrys_product_nds'));

			$requestData = array(
					'RequestId'             => $checkId,
					'Password'              => intval($this->config->get('starrys_cashier_password')),
					'PhoneOrEmail'          => $order_data['email'],
					'Group'                 => $this->config->get('starrys_group'),
					'FullResponse'          => false,
					'Cash'                  => 0,
					'Device'                => 'auto',
					'NonCash'               => array(0, round($order_data['total'] * 100), 0),
					'MaxDocumentsInTurn'    => intval($this->config->get('starrys_docs_in_turn')),
					'TaxMode'               => pow(2, $this->config->get('starrys_tax_mode')),
					'DocumentType'			=> $documentType,
					'Lines'                 => array()
				);
			if($this->config->get('starrys_client_id')) {
				$requestData['ClientId'] = $this->config->get('starrys_client_id');
			}

			$total_sum = 0;
			$iterator = 0;

			$lines = new Starrys\Cashbox\Api\Productlist($order_data['total']);

			foreach($products as $product) {
				$lines->addItem($product['name'], $product['price'], $vat, $product['quantity']);
			}

			if ($shipping > 0) {
				$lines->addItem($order_data['shipping_method'], $shipping, $this->config->get('starrys_delivery_nds'));
			}
			$requestData['Lines'] = $lines->getItems();

			$api = $this->init_api();
			$result = $api->sendRequest('Complex', $requestData);

			if($this->config->get('starrys_log')) {
				$this->writeLog('request', $api);
				$this->writeLog('response', $result);
			}
			if($result && $result->response && $result->response->Response) {

				if($result->response->Response->Error == 0) {
					$this->db->query("update " . DB_PREFIX . "starrys_receipt set is_print=1 where id=".$checkId);
				} else {
					$this->writeLog('error', $result);
				}
			}
		}
	}

	public function init_api() {
		$api = new Starrys\Cashbox\Api\Client(
			trim($this->config->get('starrys_endpoint')),
			trim($this->config->get('starrys_group')),
			"auto"
			);

		$api->setCmsId('Opencart '.VERSION, $this->getVersion());
		if ($this->config->get('starrys_client_id')) {
			$api->setClientId(trim($this->config->get('starrys_client_id')));
		}
		$api->setPasswords($this->config->get('starrys_cashier_password'));

		if ($this->config->get('starrys_cert_path')) {
			$cert_path = $_SERVER["DOCUMENT_ROOT"].'/'.trim($this->config->get('starrys_cert_path')," \t\n\r\0\x0B/");
			$key_path = $_SERVER["DOCUMENT_ROOT"].'/'.trim($this->config->get('starrys_key_path')," \t\n\r\0\x0B/");
			$api->setCertificate($cert_path, $key_path, trim($this->config->get('starrys_cert_password')));
		}
		return $api;
	}

	private function getCheckId($order_id, $type = "sell") {
		$res = $this->db->query("SELECT id FROM " . DB_PREFIX . "starrys_receipt where order_id=".$order_id." and type='".$this->db->escape($type)."'");
		if($res->num_rows){
			return strval($res->row['id']);
		} else {
			$this->db->query("INSERT INTO " . DB_PREFIX . "starrys_receipt (order_id, type, date_added) values ({$order_id},'$type',".time().")");
			return strval($this->db->getLastId());
		}
	}

	private function getOrderProducts($order_id) {
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order_product WHERE order_id = '" . (int)$order_id . "'");

		return $query->rows;
	}

	private function getOrderTotals($order_id) {
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order_total WHERE order_id = '" . (int)$order_id . "' ORDER BY sort_order");

		return $query->rows;
	}

	public function writeLog($type, $data) {
		@file_put_contents(DIR_LOGS.'/starrys_'.$type.'_'.date('Y-m-d').'.log', var_export($data, true), FILE_APPEND);
	}

	public function getVersion() {
		$query = $this->db->query("SELECT version FROM " . DB_PREFIX . "modification WHERE code = 'checkonline'");
		return $query->row['version'];
	}

}